/**
 * Copyright (c) 2011 Bruno Jouhier <bruno.jouhier@sage.com>
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */
"use strict";
var fs = require("fs");
var path = require("path");
var transform = require("./transform");
var compile = require("./compile");
var registered = false;

/// !doc
/// 
/// # streamline/lib/compiler/register
///  
/// Streamline `require` handler registration
/// 
/// * `register.register(options)`  
///   Registers `require` handlers for streamline.  
///   `options` is a set of default options passed to the `transform` function.
exports.register = function(options) {
	if (registered)
		return;
	registered = true;

	require.extensions[".js"] = function(module, filename) {
		var content = compile.loadFileSync(filename, options);
		module._compile(content, filename);
	};

	// we'll also override .coffee registration to support streamlining, but
	// only if .coffee has already been registered. if not, nothing to do:
	if (!require.extensions['.coffee'])
		return;

	// .coffee has been registered, so we'll include coffee-script, but be
	// robust to failure (this is an edge case though).
	try {
		var coffee = require('coffee-script');
	} catch (ex) {
		return;
	}

	var origCoffeeRequire = require.extensions['.coffee'];
	require.extensions['.coffee'] = function(module, filename) {
		// we'll only handle files that are named in the streamline style:
		if (!filename.match(/_\.coffee$/)) {
			return origCoffeeRequire(module, filename);
		}

		var content = fs.readFileSync(filename, 'utf8');

		// coffee compiles by default with a function wrapper. this breaks
		// streamline since the function wrapper doesn't have the necessary
		// callback parameter. we could manually add it (e.g. using regex),
		// but the good news is we don't need coffee's function wrapper,
		// since streamline adds its own, so we tell coffee not to add it.
		var compiled = coffee.compile(content, {
			bare: true
		});

		// support the same file-level options that we support in JS, e.g. a
		// custom callback parameter. we unfortunately need to duplicate this
		// logic that's already in transform() because the JS generated by
		// coffee doesn't include single-line comments (e.g. # foo).
		var options;
		var sourceOptions = /streamline\.options\s*=\s*(\{.*\})/.exec(content);
		if (sourceOptions) {
			options = JSON.parse(sourceOptions[1]);
		} else {
			options = {};
		}

		// either way, default to "lines preserve" since we're require()-ing.
		options.lines = options.lines || "preserve";

		module._compile(transform.transform(compiled, options), filename);
	};
};

